#!/usr/bin/env python

#  CountdownScreenlet (c) Patrik Kullman 2007 <patrik@yes.nu>
#
# INFO:
# - Count down to a specific event
# 
# TODO:
# -  beautify
#
# CHANGELOG:
# v0.2:
# - centered text
# - display the event text instead of a negative countdown
# 
# v0.1:
# - initial version

import screenlets
from screenlets.options import StringOption
import cairo
import gtk
import pango
import time
from os import system
import gobject

class CountdownScreenlet(screenlets.Screenlet):
	
	# default meta-info for Screenlets
	__name__ = 'CountdownScreenlet'
	__version__ = '0.2'
	__author__ = 'Patrik Kullman'
	__desc__ = 'Display time left to a specific date and time'

	# internals
	__timeout = None

	# editable options and defaults
	__update_interval = 1 # every second
	text = "New Year"
	to_date = "2008-01-01 00:00:00"
		
	# constructor
	def __init__(self, **keyword_args):
		# call super
		screenlets.Screenlet.__init__(self, width=320, height=110, **keyword_args)
		# set theme
		self.theme_name = "default"
		# add default menu items
		self.add_default_menuitems()
		# add option groups
		self.add_options_group('Countdown', 'Countdown settings')
		self.add_option(StringOption('Countdown',
			'text', 						# attribute-name
			self.text,						# default-value
			'Countdown text', 						# widget-label
			'Mihin lasketaan aikaa?' # description
			))
		self.add_option(StringOption('Countdown',
			'to_date', 						# attribute-name
			self.to_date,						# default-value
			'', 						# widget-label
			'The DateTime (YYYY-mm-dd H:i:s) to count down to' # description
			))
		self.__timeout = gobject.timeout_add(self.__update_interval * 1000, self.update)

	def __setattr__(self, name, value):
		# call Screenlet.__setattr__ in baseclass (ESSENTIAL!!!!)
		screenlets.Screenlet.__setattr__(self, name, value)
		# redraw the canvas when server-info is changed since a server host/ip addition/removal will change the size of the screenlet

	def update(self):
		gobject.idle_add(self.redraw_canvas)
		return True

	def on_draw(self, ctx):
		self.__timeleft = self.get_time_left(self.to_date)
		# scaling above 500% is very slow and might brake it (why?)
		if self.scale > 5:
			self.scale = 5
		# if theme is loaded
		if self.theme:
			# find out how many servers that are configured and force a update_shape() when they're changed from the last draw.
			# apparently the settings doesn't get loaded until the widget is first displayed, and no update_shape is performed at
			# that point
			# make sure that the background covers all the icons
			ctx.save()
			ctx.scale(self.scale * 1.0, self.scale)
			ctx.translate(0, 0)
			self.theme.render(ctx, 'background')
			ctx.restore()

			ctx.save()
			ctx.scale(self.scale, self.scale)
			self.draw_text(ctx, "" + self.text, 32, 24, 12) #oikealta, ylhaalta, fontin koko
			self.draw_text(ctx, self.__timeleft, 33, 51, 11) #oikealta, ylhaalta, fontin koko
			ctx.restore()

			ctx.save()
			ctx.scale(self.scale * 1.5, self.scale)
			ctx.translate(0, 0)
			self.theme.render(ctx, 'glass')
			ctx.restore()

	def on_draw_shape(self, ctx):
		if self.theme:
			self.on_draw(ctx)

	def draw_text(self, ctx, value, x, y, size):
		# stolen from CalcScreenlet ;)
		ctx.save()
		ctx.translate(x, y)
		p_layout = ctx.create_layout()
		p_fdesc = pango.FontDescription()
		p_fdesc.set_family_static("Garamond")
		p_fdesc.set_size(size * pango.SCALE)
		p_layout.set_font_description(p_fdesc)
		p_layout.set_width(320 * pango.SCALE)
		p_layout.set_alignment(pango.ALIGN_CENTER)
		p_layout.set_markup(value)
		ctx.set_source_rgba(1, 1, 4, 0.5)
		ctx.show_layout(p_layout)
		p_layout.set_alignment(pango.ALIGN_LEFT)
		ctx.restore()

	def get_time_left(self, date):
		try:
			then = time.mktime(time.strptime(date, "%Y-%m-%d %H:%M:%S"))
			now = time.time()
			seconds_left = then - now
			if (seconds_left > 0):
				days_left = int(seconds_left / 60 / 60 / 24)
				seconds_left = seconds_left - (days_left * 24 * 60 * 60)
				hours_left = int(seconds_left / 60 / 60)
				seconds_left = seconds_left - (hours_left * 60 * 60)
				minutes_left = int(seconds_left / 60)
				seconds_left = int(seconds_left - (minutes_left * 60))
				return str(days_left) + "      " + str(hours_left) + "      " + str(minutes_left) + "     " + str(seconds_left) + ""
			else:
				#return self.text + "!"
				return " 00     00     00     00"
		except ValueError:
			return "Invalid time format"
	
# If the program is run directly or passed as an argument to the python
# interpreter then create a Screenlet instance and show it
if __name__ == "__main__":
	import screenlets.session
	screenlets.session.create_session(CountdownScreenlet)
